#include "platform.h"
#include "error_handler.h"

void SystemClock_Config(void) {
	RCC_OscInitTypeDef RCC_OscInitStruct;
	RCC_ClkInitTypeDef RCC_ClkInitStruct;
	RCC_PeriphCLKInitTypeDef PeriphClkInit;

	// Initializes the CPU, AHB and APB busses clocks
	RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI | RCC_OSCILLATORTYPE_HSI14;
	RCC_OscInitStruct.HSIState = RCC_HSI_ON;
	RCC_OscInitStruct.HSI14State = RCC_HSI14_ON;
	RCC_OscInitStruct.HSICalibrationValue = 16;
	RCC_OscInitStruct.HSI14CalibrationValue = 16;
	RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
	RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
	RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL12;
	RCC_OscInitStruct.PLL.PREDIV = RCC_PREDIV_DIV1;
	if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
		Error_Handler();

	// Initializes the CPU, AHB and APB busses clocks
	RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_SYSCLK	| RCC_CLOCKTYPE_PCLK1;
	RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
	RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
	RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
	if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1) != HAL_OK)
		Error_Handler();

	PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART1 | RCC_PERIPHCLK_I2C1;
	PeriphClkInit.Usart1ClockSelection = RCC_USART1CLKSOURCE_PCLK1;
	PeriphClkInit.I2c1ClockSelection = RCC_I2C1CLKSOURCE_HSI;
	if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
		Error_Handler();

	// Configure the Systick interrupt time
	HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq() / 1000); // 1ms

	// Configure the Systick
	HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

	/* SysTick_IRQn interrupt configuration */
	HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

static void GPIO_Init(void) {
	GPIO_InitTypeDef GPIO_InitStruct;

	// Enable GPIO clock
	__HAL_RCC_GPIOA_CLK_ENABLE();
	__HAL_RCC_GPIOB_CLK_ENABLE();

	// TIM1
	// PB0     ------> TIM1_CH2N
    // PA9     ------> TIM1_CH2
    GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
    GPIO_InitStruct.Alternate = GPIO_AF2_TIM1;

    GPIO_InitStruct.Pin = GPIO_PIN_9;
    HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
    GPIO_InitStruct.Pin = GPIO_PIN_0;
    HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

    // TIM16
    // PA6     ------> TIM16_CH1
    // PB6     ------> TIM16_CH1N
    GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;

    GPIO_InitStruct.Pin = GPIO_PIN_6;
    GPIO_InitStruct.Alternate = GPIO_AF5_TIM16;
    HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
    GPIO_InitStruct.Pin = GPIO_PIN_6;
    GPIO_InitStruct.Alternate = GPIO_AF2_TIM16;
    HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

    // TIM17
    // PA7     ------> TIM17_CH1
    // PB7     ------> TIM17_CH1N
    GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;

    GPIO_InitStruct.Pin = GPIO_PIN_7;
    GPIO_InitStruct.Alternate = GPIO_AF5_TIM17;
    HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
    GPIO_InitStruct.Pin = GPIO_PIN_7;
    GPIO_InitStruct.Alternate = GPIO_AF2_TIM17;
    HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
}

static void Timer14_Init(void) {
	__HAL_RCC_TIM14_CLK_ENABLE();

	TIM14->CR1 = 0;
	TIM14->DIER = TIM_DIER_UIE;

	TIM14->ARR = TIM_ARR_PHSYNC;	// Auto-reload value : 300kHz overflow
	TIM14->PSC = 0;

	NVIC_EnableIRQ(TIM14_IRQn);
}

static void Timer1_Init(void) {
	__HAL_RCC_TIM1_CLK_ENABLE();

	TIM1->CR1 = 0;
	TIM1->CR2 = 0;
	TIM1->DIER = 0;

	TIM1->ARR = TIM_ARR_SW;	// Auto-reload value : 100kHz overflow
	TIM1->PSC = 0;
	TIM1->CCR2 = 0;

	TIM1->CCMR1 = 0x6800;	// 110: PWM mode 1, up counting.
	TIM1->CCER = 0x0050; 	// CH2 output enable with complement, active high

	TIM1->BDTR = TIM_BDTR_DTG_Msk & TIM_DEADTIME;	// Set dead-time
}

static void Timer16_Init(void) {
	__HAL_RCC_TIM16_CLK_ENABLE();

	TIM16->CR1 = 0;
	TIM16->CR2 = 0;
	TIM16->DIER = 0;

	TIM16->ARR = TIM_ARR_SW;		// Auto-reload value : 100kHz overflow
	TIM16->PSC = 0;
	TIM16->CCR1 = 0;

	TIM16->CCMR1 = 0x0068;	// 110: PWM mode 1, up counting.
	TIM16->CCER = 0x0005; 	// CH1 output enable with complement, active high

	TIM16->BDTR = TIM_BDTR_DTG_Msk & TIM_DEADTIME;	// Set dead-time
}

static void Timer17_Init(void) {
	__HAL_RCC_TIM17_CLK_ENABLE();

	TIM17->CR1 = 0;
	TIM17->CR2 = 0;
	TIM17->DIER = 0;

	TIM17->ARR = TIM_ARR_SW;		// Auto-reload value : 100kHz overflow
	TIM17->PSC = 0;
	TIM17->CCR1 = 0;

	TIM17->CCMR1 = 0x0068;	// 110: PWM mode 1, up counting.
	TIM17->CCER = 0x0005; 	// CH1 output enable with complement, active high

	TIM17->BDTR = TIM_BDTR_DTG_Msk & TIM_DEADTIME;	// Set dead-time
}

static void Timer_Sync_1_16_17(void) {
	extern volatile uint32_t* tim_cr1;

	// Clear counter value
	TIM14->CNT = 0;
	TIM1->CNT = 0;
	TIM16->CNT = 0;
	TIM17->CNT = 0;

	// Enable the synchronizer timer
	TIM14->CR1 |= 1;

	// Wait until the synchronization is done
	while (tim_cr1 > 0);

	// Reconfigure timer14 for executing control routine
	TIM14->CR1 = 0;
	TIM14->DIER = TIM_DIER_UIE;

	TIM14->ARR = TIM_ARR_CON;		// Auto-reload value : 50kHz overflow
	TIM14->PSC = 0;
	TIM14->CNT = 0;

	// Enable the timer again
	TIM14->CR1 |= TIM_CR1_CEN;
}

void hw_init(void) {
	/* Reset of all peripherals, Initializes the Flash interface and the Systick. */
	HAL_Init();

	SystemClock_Config();
	GPIO_Init();

	Timer1_Init();
	Timer16_Init();
	Timer17_Init();
	Timer14_Init();
	Timer_Sync_1_16_17();
}
